%*****************************************************
% Test des angles d'Euler et de calcul de quaternions
%*****************************************************
%
% Auteur: J. Dezert
% Date : 10 Avril, 2001
% Email : Jean.Dezert@onera.fr
%
% participant: guiziou Robert, guiziouRob@aol.com

% ******************************************************
clear all
clc

deg_rad=pi/180;
rad_deg=180/pi;

% Vecteurs de la base fixe ECEF
%------------------------------
Ox_in_ECEF=[1 0 0]';
Oy_in_ECEF=[0 1 0]';
Oz_in_ECEF=[0 0 1]';

% Choix des angles d'Euler classiques
%------------------------------------
% 1re rotation  Psi   -> Rotation autour de Oz   --> repre (Ox',Oy',Oz)
% 2ime rotation Theta -> Rotation autour de Ox'  --> repre (Ox',Oy'',Oz')
% 3ime rotation Phi   -> Rotation autour de Oz' --> repre final (Ox'',Oy''',Oz')
%
Psi_deg=35;
Theta_deg=-70;
Phi_deg=10;


Psi=Psi_deg*deg_rad;
Theta=Theta_deg*deg_rad;
Phi=Phi_deg*deg_rad;

% Calcul de la matrice de Rotation globale
%-----------------------------------------
CP=cos(Psi);SP=sin(Psi);
CT=cos(Theta);ST=sin(Theta);
CF=cos(Phi);SF=sin(Phi);

% MA MATRICE DE ROTATION OU DE PASSAGE Pmat ANCIENNE BASE VERS NOUVELLE BASE

Rot_Mat=[ CF*CP-CT*SP*SF    -CP*SF-CF*CT*SP   ST*SP;
          CF*SP+CP*CT*SF   -SF*SP+CF*CT*CP    -ST*CP;
          SF*ST             ST*CF                 CT];


Rot_Mat0=Rot_Mat;

% Coordonnes des axes du nouveau repre dans le repre initial
%--------------------------------------------------------------
ROx_in_ECEF=Rot_Mat*Ox_in_ECEF; % vecteur Ox apres rotation globale
ROy_in_ECEF=Rot_Mat*Oy_in_ECEF; % vecteur Oy apres rotation globale
ROz_in_ECEF=Rot_Mat*Oz_in_ECEF; % vecteur Oz apres rotation globale

Nouveau_repere=[ROx_in_ECEF ROy_in_ECEF ROz_in_ECEF];

% Naturellement on retrouve la matrice de passage Rot_Mat=Nouveau_repere

% Calcul du quaternion  partir de la matrice de rotation (code R.
% Chiavassa dbugu)
%-------------------------------------------------------------------------------

% Note : La matrice de rotation s'obtient  partir du quaternion associ par
%
%        Rot_MatQBC=[2*(q0^2+q1^2)-1    2*(q1*q2-q0*q3)      2*(q1*q3+q0*q2);
%                    2*(q1*q2+q0*q3)    2*(q0^2+q2^2)-1      2*(q3*q2-q0*q1);
%                    2*(q1*q3-q0*q2)    2*(q2*q3+q0*q1)      2*(q0^2+q3^2)-1];
%
Pmat=Rot_Mat;
Prec=1.e-8;
trac=trace(Pmat);
if (1+trac)>0,
    qt00=(1+trac)^0.5/2;
    else
    qt00=0; 
end
if (1-trac+2*Pmat(1,1))>0,
   qt10=(1-trac+2*Pmat(1,1))^0.5/2;
   else
   qt10=0;
end
if(1-trac+2*Pmat(2,2))>0,
   qt20=(1-trac+2*Pmat(2,2))^0.5/2;
   else
   qt20=0;
end
if (1-trac+2*Pmat(3,3))>0,
    qt30=(1-trac+2*Pmat(3,3))^0.5/2;
    else
    qt30=0;
end
if (abs(qt00)>Prec),
   qt10=qt10*sign((Pmat(3,2)-Pmat(2,3))/4/qt00);
   qt20=qt20*sign((Pmat(1,3)-Pmat(3,1))/4/qt00);
   qt30=qt30*sign((Pmat(2,1)-Pmat(1,2))/4/qt00);
   elseif (abs(qt10)>Prec),
       qt20=qt20*sign((Pmat(1,2)+Pmat(2,1))/4/qt10);
       qt30=qt30*sign((Pmat(1,3)+Pmat(3,1))/4/qt10);
       qt00=qt00*sign((Pmat(3,2)-Pmat(2,3))/4/qt10);
   elseif (abs(qt20)>Prec),
       qt10=qt10*sign((Pmat(1,2)+Pmat(2,1))/4/qt20);
       qt30=qt30*sign((Pmat(2,3)+Pmat(3,2))/4/qt20);
       qt00=qt00*sign((Pmat(1,3)-Pmat(3,1))/4/qt20);
   elseif (abs(qt30)>Prec),
       qt20=qt20*sign((Pmat(3,2)+Pmat(2,3))/4/qt30);
       qt10=qt10*sign((Pmat(1,3)+Pmat(3,1))/4/qt30);
       qt00=qt00*sign((Pmat(2,1)-Pmat(1,2))/4/qt30);
end

% Quaternion calcul  partir de cette matrice de rotation

Quaternion0=[qt00 qt10 qt20 qt30];

% Vrification du calcul direct du quaternion  partir des angles d'euler

% Quaternion de la rotation Phi

Quat_Phi=[cos(Phi/2) sin(Phi/2)*[Rot_Mat(1,3) Rot_Mat(2,3) Rot_Mat(3,3)]];

% Quaternion de la rotation Theta

Quat_Theta=[cos(Theta/2) sin(Theta/2)*[cos(Psi) sin(Psi) 0]];

% Quaternion de la rotation Psi

Quat_Psi=[cos(Psi/2) sin(Psi/2)*[0 0 1]];

% On effectue le produit des 3 quaternions dans le bon ordre en
% utilisant la fonction produit ProdQuat(q,Q)

produit1=ProdQuat(Quat_Phi,Quat_Theta);
produit2=ProdQuat(produit1,Quat_Psi);

% QUATERNION PRODUIT DES 3 QUATERNIONS:

Quaternion=produit2;
Quaternion1=produit2;
q0=Quaternion(1);q1=Quaternion(2);q2=Quaternion(3);q3=Quaternion(4);

% Matrice de rotation associe  ce quaternion

Rot_Mat2=[q0^2+q1^2-q2^2-q3^2   2*(q1*q2-q0*q3)      2*(q1*q3+q0*q2);
         2*(q1*q2+q0*q3)       q0^2+q2^2-q3^2-q1^2  2*(q3*q2-q0*q1);
         2*(q1*q3-q0*q2)       2*(q2*q3+q0*q1)      q0^2+q3^2-q1^2-q2^2];

Rot_Mat2=Rot_Mat2;

comparaison=norm(Rot_Mat0-Rot_Mat2)*norm(Rot_Mat0+Rot_Mat2)/4;

% Test russi, la comparaison le prouve  quelques 1E-16 prs

% NB: Avec le quaternion rsultat du produit, vous retrouvez naturellement

% Les lments de la rotation globale: AXE et ANGLE'

disp('   Quaternion calcul avec la matrice de passage');Quaternion0,
disp('   Quaternion calcul avec le produit des quaternions');Quaternion1,
disp('   Matrice de passage initiale');Rot_Mat0,
disp('   Matrice de passage calcule avec le quaternion produit');Rot_Mat2
disp('   Test de comparaison des matrices');comparaison,

% CALCUL DES ELEMENTS DE LA ROTATION GLOBALE, Axe et angle

disp('   La rotation globale a pour axe le vecteur Axe')

Axe1=[q1 q2 q3]';Axe=Axe1/norm(Axe1)

disp('   L''angle de rotation est TETA en degrs')

TETA=2*acos(q0)*rad_deg

Quat_Psi,Quat_Theta,Quat_Phi,

end.
